import { Injectable, Logger } from '@nestjs/common';
import { IChannel } from '../interfaces/channel.interface';
import { IClient } from '../interfaces/client.interface';
import { IPopulatedChannel } from '../interfaces/populatedchannel.interface';
import { TeamSpeakClient } from 'node-ts';

const env = process.env;

/* eslint-disable camelcase */
/* eslint-disable @typescript-eslint/no-explicit-any */
/* eslint-disable @typescript-eslint/naming-convention */
// Disable specific ESLint rules because node-ts has no typings
@Injectable()
export class TeamspeakService {
    private client: TeamSpeakClient;

    constructor() {
        this.init();
    }

    async init(): Promise<void> {
        this.client = new TeamSpeakClient(env.TEAMSPEAK_HOST);

        try {
            await this.client.connect();
            await this.client.send('use', { sid: 1 });
            await this.client.send('login', {
                client_login_name: env.TEAMSPEAK_QUERY_NAME,
                client_login_password: env.TEAMSPEAK_QUERY_PASS,
            });
        } catch (err) {
            Logger.error('Error initializing TeamSpeak Server connection', err, 'TeamspeakService');
        }
    }

    async getClients(): Promise<IClient[]> {
        const clients: IClient[] = [];
        const query = await this.client.send('clientlist', { });
        const rawList: any = query.response;
        const clientList = rawList.filter(el => el.client_type == 0);
        for(const client of clientList)
        {
            clients.push({ id: client.clid, name: client.client_nickname, channel: client.cid });
        }
        return clients;
    }

    async getChannels(): Promise<IChannel[]> {
        const channels: IChannel[] = [];
        const query = await this.client.send('channellist', {}, []);
        const channelList: any = query.response;
        for(const channel of channelList) {
            if(channel.pid == 0) {
                channels.push({ id: channel.cid, children: [], name: channel.channel_name });
            } else {
                channels.find(el => el.id == channel.pid).children.push({ id: channel.cid, children: [], name: channel.channel_name });
            }
        }
        return channels;
    }

    async getPopulatedChannelList(): Promise<IPopulatedChannel[]> {
        const channels = await this.getChannels();
        const clients = await this.getClients();
        return this.addClientToChannel(channels, clients);
    }

    private addClientToChannel(channels, clients): IPopulatedChannel[] {
        const channelList: IPopulatedChannel[] = [];
        for(const channel of channels) {
            const populatedChannel: IPopulatedChannel = channel;
            populatedChannel.clients = clients.filter(el=> el.channel == channel.id);
            if(channel.children.length > 0) {
                this.addClientToChannel(channel.children, clients);
            }
            channelList.push(populatedChannel);
        }
        return channelList;
    }

}